package de.jottyfan.bico.modules.camp;

import static de.jottyfan.bico.db.camp.Tables.T_AGE;
import static de.jottyfan.bico.db.camp.Tables.T_REGISTRATION;

import java.time.LocalDateTime;
import java.util.List;

import org.jooq.DSLContext;
import org.jooq.DeleteConditionStep;
import org.jooq.InsertValuesStep20;
import org.jooq.Record19;
import org.jooq.SelectConditionStep;
import org.jooq.SelectOnConditionStep;
import org.jooq.SelectSeekStep1;
import org.jooq.UpdateConditionStep;
import org.jooq.impl.DSL;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import de.jottyfan.bico.Main;
import de.jottyfan.bico.db.camp.enums.EnumCamp;
import de.jottyfan.bico.db.camp.enums.EnumSex;
import de.jottyfan.bico.db.camp.tables.records.TAgeRecord;
import de.jottyfan.bico.db.camp.tables.records.TRegistrationRecord;
import de.jottyfan.bico.modules.camp.model.RegistrationBean;
import jakarta.validation.Valid;

/**
 *
 * @author jotty
 *
 */
@Repository
public class RegistrationRepository {
	@Autowired
	private DSLContext jooq;

	/**
	 * save the validated bean
	 *
	 * @param bean        the bean
	 * @param registrator the principal name
	 */
	public void save(@Valid RegistrationBean bean, String registrator) {
		jooq.transaction(t -> {
			if (bean.getPkRegistration() == null) {
				InsertValuesStep20<TRegistrationRecord, EnumCamp, String, String, String, EnumSex, Boolean, String, Integer, Boolean, String, Boolean, Integer, Boolean, Boolean, Boolean, Boolean, Boolean, Boolean, Boolean, Boolean> sql = DSL.using(t)
				// @formatter:off
					.insertInto(T_REGISTRATION,
							        T_REGISTRATION.CAMP,
							        T_REGISTRATION.REGISTRATOR,
							        T_REGISTRATION.FORENAME,
							        T_REGISTRATION.SURNAME,
							        T_REGISTRATION.SEX,
							        T_REGISTRATION.BARRIER_FREE,
							        T_REGISTRATION.NUTRITION,
							        T_REGISTRATION.DRIVER_PROVIDE_PLACES,
							        T_REGISTRATION.WANT_PLACE_IN_CAR,
							        T_REGISTRATION.DISEASES,
							        T_REGISTRATION.REQUIRE_PAYMENT,
							        T_REGISTRATION.FK_AGE,
							        T_REGISTRATION.DAY0,
							        T_REGISTRATION.DAY1,
							        T_REGISTRATION.DAY2,
							        T_REGISTRATION.DAY3,
							        T_REGISTRATION.DAY4,
							        T_REGISTRATION.TOWELS,
							        T_REGISTRATION.BED_LINEN,
							        T_REGISTRATION.COT)
					.values(EnumCamp.Gemeindefreizeit_2025, registrator, bean.getForename(), bean.getSurname(), EnumSex.lookupLiteral(bean.getSex()),
							bean.getBarrierFree(), bean.getNutrition(), bean.getDriverProvidePlaces(), bean.getWantPlaceInCar(), bean.getDiseases(), bean.getRequirePayment(),
							bean.getFkAge(), bean.getDay0(), bean.getDay1(), bean.getDay2(), bean.getDay3(), bean.getDay4(), bean.getTowels(), bean.getBedLinen(), bean.getCot());
				// @formatter:on
				Main.LOGGER.trace(sql);
				sql.execute();
			} else {
				UpdateConditionStep<TRegistrationRecord> sql = DSL.using(t)
				// @formatter:off
					.update(T_REGISTRATION)
					.set(T_REGISTRATION.FORENAME, bean.getForename())
					.set(T_REGISTRATION.SURNAME, bean.getSurname())
					.set(T_REGISTRATION.SEX, EnumSex.lookupLiteral(bean.getSex()))
					.set(T_REGISTRATION.REGISTRATOR, registrator)
					.set(T_REGISTRATION.BARRIER_FREE, bean.getBarrierFree())
					.set(T_REGISTRATION.NUTRITION, bean.getNutrition())
			    .set(T_REGISTRATION.DRIVER_PROVIDE_PLACES, bean.getDriverProvidePlaces())
			    .set(T_REGISTRATION.WANT_PLACE_IN_CAR, bean.getWantPlaceInCar())
			    .set(T_REGISTRATION.DISEASES, bean.getDiseases())
			    .set(T_REGISTRATION.REQUIRE_PAYMENT, bean.getRequirePayment())
			    .set(T_REGISTRATION.FK_AGE, bean.getFkAge())
			    .set(T_REGISTRATION.DAY0, bean.getDay0())
			    .set(T_REGISTRATION.DAY1, bean.getDay1())
			    .set(T_REGISTRATION.DAY2, bean.getDay2())
			    .set(T_REGISTRATION.DAY3, bean.getDay3())
			    .set(T_REGISTRATION.DAY4, bean.getDay4())
			    .set(T_REGISTRATION.TOWELS, bean.getTowels())
			    .set(T_REGISTRATION.BED_LINEN, bean.getBedLinen())
			    .set(T_REGISTRATION.COT, bean.getCot())
					.where(T_REGISTRATION.PK_REGISTRATION.eq(bean.getPkRegistration()));
				// @formatter:on
				Main.LOGGER.trace(sql);
				sql.execute();
			}
		});
	}

	/**
	 * get all registrations (for admins only)
	 *
	 * @return the registrations
	 */
	public List<RegistrationBean> getAllRegistrations() {
		SelectSeekStep1<TRegistrationRecord, LocalDateTime> sql = jooq
		// @formatter:off
			.selectFrom(T_REGISTRATION)
			.orderBy(T_REGISTRATION.CREATED);
		// @formatter:on
		Main.LOGGER.trace(sql);
		return sql.fetchInto(RegistrationBean.class);
	}

	/**
	 * get all registrations of name
	 *
	 * @param name the name
	 * @return the registrations
	 */
	public List<RegistrationBean> getRegistrations(String name) {
		SelectConditionStep<TRegistrationRecord> sql = jooq
		// @formatter:off
			.selectFrom(T_REGISTRATION)
			.where(T_REGISTRATION.REGISTRATOR.eq(name));
		// @formatter:on
		Main.LOGGER.trace(sql);
		return sql.fetchInto(RegistrationBean.class);
	}

	/**
	 * get the registration of id if the creator is the name; null else
	 *
	 * @param id   the id
	 * @param name the name
	 * @return the bean or null
	 */
	public RegistrationBean getBean(Integer id, String name) {
		SelectConditionStep<TRegistrationRecord> sql = jooq
		// @formatter:off
			.selectFrom(T_REGISTRATION)
			.where(T_REGISTRATION.PK_REGISTRATION.eq(id))
			.and(T_REGISTRATION.REGISTRATOR.eq(name));
		// @formatter:on
		Main.LOGGER.trace(sql);
		return sql.fetchOneInto(RegistrationBean.class);
	}

	/**
	 * delete the registration if the name is the creator
	 *
	 * @param id   the id
	 * @param name the name
	 */
	public void delete(Integer id, String name) {
		DeleteConditionStep<TRegistrationRecord> sql = jooq
		// @formatter:off
			.deleteFrom(T_REGISTRATION)
			.where(T_REGISTRATION.PK_REGISTRATION.eq(id))
			.and(T_REGISTRATION.REGISTRATOR.eq(name));
		// @formatter:on
		Main.LOGGER.trace(sql);
		sql.execute();
	}

	/**
	 * get all ages
	 *
	 * @return the ages
	 */
	public List<TAgeRecord> getAges() {
		SelectSeekStep1<TAgeRecord, Integer> sql = jooq
		// @formatter:off
			.selectFrom(T_AGE)
			.orderBy(T_AGE.PK_AGE);
	  // @formatter:on
		Main.LOGGER.trace(sql);
		return sql.fetchInto(TAgeRecord.class);
	}

	/**
	 * get CSV version of the list, only for admins
	 *
	 * @return the csv
	 */
	public String getDownload() {
		SelectOnConditionStep<Record19<LocalDateTime, String, String, EnumSex, String, Boolean, Boolean, Boolean, Boolean, Boolean, Boolean, Boolean, Boolean, Boolean, Boolean, String, String, Integer, Boolean>> sql = jooq
		// @formatter:off
		  .select(T_REGISTRATION.CREATED,
		  		    T_REGISTRATION.FORENAME,
		  		    T_REGISTRATION.SURNAME,
		  		    T_REGISTRATION.SEX,
		  		    T_AGE.NAME,
		  		    T_REGISTRATION.DAY0,
		  		    T_REGISTRATION.DAY1,
		  		    T_REGISTRATION.DAY2,
		  		    T_REGISTRATION.DAY3,
		  		    T_REGISTRATION.DAY4,
		  		    T_REGISTRATION.BARRIER_FREE,
		  		    T_REGISTRATION.TOWELS,
		  		    T_REGISTRATION.BED_LINEN,
		  		    T_REGISTRATION.COT,
		  		    T_REGISTRATION.REQUIRE_PAYMENT,
		  		    T_REGISTRATION.DISEASES,
		  		    T_REGISTRATION.NUTRITION,
		  		    T_REGISTRATION.DRIVER_PROVIDE_PLACES,
		  		    T_REGISTRATION.WANT_PLACE_IN_CAR)
		  .from(T_REGISTRATION)
		  .leftJoin(T_AGE).on(T_AGE.PK_AGE.eq(T_REGISTRATION.FK_AGE));
		// @formatter:on
		Main.LOGGER.trace(sql);
		return sql.fetch().formatCSV(true);
	}
}
